% For details, please see the main text.
clear
close all

data = readtable('Experimental_results.xlsx');

% Crack-free parameters
Vp0_basalt  = 6.8973; Vs0_basalt  = 3.8196; rho0_basalt  = 2.9285; % Basalt
Vp0_diabase = 6.9073; Vs0_diabase = 3.8325; rho0_diabase = 2.9415; % Diabase
Vp0_gabbro  = 6.9135; Vs0_gabbro  = 3.8249; rho0_gabbro  = 2.9276; % Gabbro
Vp0_oxgab   = 6.8753; Vs0_oxgab   = 3.7812; rho0_oxgab   = 2.9203; % Oxide gabbro
Vp0_trond   = 6.7197; Vs0_trond   = 3.5905; rho0_trond   = 2.7596; % Trondhjemite
Vp0_diorite = 6.7222; Vs0_diorite = 3.6198; rho0_diorite = 2.8200; % Diorite
Vp0_olgab   = 6.8432; Vs0_olgab   = 3.7522; rho0_olgab   = 2.9256; % Olivine gabbro

% Setting fluid parameters
Kf = 2.2; % Bulk modulus
rhof = 1.02; % Density
Cf = 5.7; % Conductivity

% Input data
Vp_wet = [data.Vp_wet_x,data.Vp_wet_y,data.Vp_wet_z]; % [km/s]
Vs_wet = [data.Vs_wet_x,data.Vs_wet_y,data.Vs_wet_z]; % [km/s]
porosity = data.Porosity * 1e-2; % [-]
F_meas = [data.F_x,data.F_y,data.F_z]; % Formation afctor

% Inversions for each sample
x = []; Xx = []; Xy = []; Xz = [];
v = []; Vx = []; Vy = []; Vz = [];
ff = []; Fx = []; Fy = []; Fz = [];

for i = 1:height(Vp_wet)

    if data.Lithology(i) == "Basalt"
        Vp0  = Vp0_basalt;
        Vs0  = Vs0_basalt;
        rho0 = rho0_basalt;
    elseif data.Lithology(i) == "Diabase"
        Vp0  = Vp0_diabase;
        Vs0  = Vs0_diabase;
        rho0 = rho0_diabase;
    elseif data.Lithology(i) == "Gabbro"
        Vp0  = Vp0_gabbro;
        Vs0  = Vs0_gabbro;
        rho0 = rho0_gabbro;
    elseif data.Lithology(i) == "Oxide gabbro"
        Vp0  = Vp0_oxgab;
        Vs0  = Vs0_oxgab;
        rho0 = rho0_oxgab;
    elseif data.Lithology(i) == "Diorite"
        Vp0  = Vp0_diorite;
        Vs0  = Vs0_diorite;
        rho0 = rho0_diorite;
    elseif data.Lithology(i) == "Trondhjemite"
        Vp0  = Vp0_trond;
        Vs0  = Vs0_trond;
        rho0 = rho0_trond;
    elseif data.Lithology(i) == "Olivine gabbro"
        Vp0  = Vp0_olgab;
        Vs0  = Vs0_olgab;
        rho0 = rho0_olgab;
    end

    % Crack-free parameters
    v0 = (1/2 * (Vp0/Vs0).^2 - 1)./((Vp0/Vs0).^2 - 1); % Possion's ratio
    E0 = 2 * rho0 * (1 + v0) * Vs0.^2; % Youngs' muldulus
    K0 = E0/(3 * (1 - 2 * v0)); % Bulk modulus
    G0 = E0/(2 * (1 + v0)); % Shear modulus

    % For each direction
    for j = 1:3
        % If data is not measured, skip it.
        if sum(isnan([Vp_wet(i,j),Vs_wet(i,j)])) > 0
            x(j,:) = [NaN,NaN];
        else
            F = @(x) JointInversion(x,Vp_wet(i,j),Vs_wet(i,j),F_meas(i,j),E0,v0,Kf,rho0,rhof);
        
            A = [];
            b = [];
            Aeq = [];
            beq = [];
            lb = [0,0]; % Lower bound
            ub = [5,1e-1]; % Upper bound
            x0 = [1,1e-2]; % Initial values [crack density, aspect ratio]

            options = optimoptions('lsqnonlin','Algorithm','trust-region-reflective','Display','off');
            x(j,:) = lsqnonlin(F,x0,lb,ub,options);
                  
            % Forward solutions
            v(j,:) = velocity(x(j,:),E0,v0,Kf,rho0,rhof); % Velocities
            ff(j,:) = formation_factor(x(j,:)); % Formation factor

        end
    end

    Xx = [Xx;x(1,:)]; % Determined parameters (crack density, aspect ratio) for X direction
    Xy = [Xy;x(2,:)]; % Determined parameters (crack density, aspect ratio) for Y direction
    Xz = [Xz;x(3,:)]; % Determined parameters (crack density, aspect ratio) for Z direction
    
    % Forward solutions of velocities for each direction
    Vx = [Vx;v(1,:)];
    Vy = [Vy;v(2,:)];
    Vz = [Vz;v(3,:)];

    % Forward solutions of formation factor for each direction
    Fx = [Fx;ff(1,:)];
    Fy = [Fy;ff(2,:)];
    Fz = [Fz;ff(3,:)];
end


function F = JointInversion(x,Vp_wet_meas,Vs_wet_meas,F_meas,E0,v0,Kf,rho0,rhof)
% Velocity
% --------
K0 = E0/(3 * (1 - 2 * v0));
G0 = E0/(2 * (1 + v0));
h = 16/9 * (1 - v0^2)/(1 - v0/2);

cd = x(1);
ar = x(2);

dc = 9/16 * pi * (1 - 2 * v0)/(1 - v0^2) * K0/Kf * ar;

K_wet = K0/(1 + cd * h/(1 - 2 * v0) * (1 + ((1 - v0/2) * dc/(1+dc) - 1)));
G_wet = G0/(1 + cd * h/(1 + v0) * (1 + 2/5 * ((1 - v0/2) * dc/(1+dc) - 1)));

cp = pi * cd * ar;
rho_wet = rhof * cp + rho0 * (1 - cp);

Vp_wet = sqrt((K_wet + 4/3 * G_wet)/rho_wet);
Vs_wet = sqrt(G_wet/rho_wet);
% ---------

% Resistivity
% ---------
p = pi^2/4 * cd;
p(p>1) = 1;
p(p<1/3) = 1/3;
f = 1 - 4 * (sqrt(1./p - 3/4) - 1/2).^3 + 3 * (sqrt(1./p - 3/4) - 1/2).^4;

F = 1./(1/4 .* f .* cp);

% ----------

% Weights
dVp = 0.25;
dVs = 0.25;
dR = 0.5;

% Misfit function
F = [(Vp_wet_meas   - Vp_wet)/dVp;
     (Vs_wet_meas   - Vs_wet)/dVs;
     (log10(F_meas) - log10(F))/dR];

end

function V = velocity(x,E0,v0,Kf,rho0,rhof)

K0 = E0/(3 * (1 - 2 * v0));
G0 = E0/(2 * (1 + v0));
h = 16/9 * (1 - v0^2)/(1 - v0/2);

cp = pi * x(1) * x(2);

dc = 9/16 * pi * (1 - 2 * v0)/(1 - v0^2) * K0/Kf * x(2);

K = K0/(1 + x(1) * h/(1 - 2 * v0) * (1 + ((1 - v0/2) * dc/(1+dc) - 1)));
G = G0/(1 + x(1) * h/(1 + v0) * (1 + 2/5 * ((1 - v0/2) * dc/(1+dc) - 1)));

rho = rhof * cp + rho0 * (1 - cp);

V(1) = sqrt((K + 4/3 * G)/rho);
V(2) = sqrt(G/rho);
end

function F = formation_factor(x)

p = pi^2/4 * x(1);
p(p>1) = 1;
p(p<1/3) = 1/3;
f = 1 - 4 * (sqrt(1./p - 3/4) - 1/2).^3 + 3 * (sqrt(1./p - 3/4) - 1/2).^4;

cp = pi * x(1) * x(2);

F = 1./(1/4 .* f .* cp);

end