% For details, please see the main text.
clear
close all

data = readtable('Hole1256D.xlsx');

% Input data
depth = data.Depth; % Depth [mbsf]
Vp = data.Vp; % P-wave velocity [km/s]
Vs = data.Vs; % S-wave velocity [km/s]
R = data.R; % Resistivity [Ωm]
T = data.T; % Temperature [degC]
Cf = (3 + T/10); % Fluid conductivity [S/m]

% Crack-free parameters
Vp0 = 6.8957; % Vp [km/s]
Vs0 = 3.8157; % Vs [km/s]
rho0 = 2.9276; % Density [g/cm3]

v0 = (1/2 * (Vp0/Vs0).^2 - 1)./((Vp0/Vs0).^2 - 1); % Poission's ratio
E0 = 2 * rho0 * (1 + v0) * Vs0.^2; % Young's modulus
G0 = E0/(2 * (1 + v0)); % Shear modulus
K0 = E0/(3 * (1 - 2 * v0)); % Bulk modulus
Kf = 2.2; % Fluid bulk modulus
rhof = 1.02; % Fluid density

X = [];
V_forward = [];
R_forward = [];

for i = 1:height(Vp)
    
    F = @(x) JointInvresion(x,Vp(i),Vs(i),R(i),E0,v0,Kf,rho0,rhof,Cf(i));

    A = [];
    b = [];
    Aeq = [];
    beq = [];

    lb = [0,0];
    ub = [inf,1e-1];   

    x0 = [1,1e-3];

    options = optimoptions('lsqnonlin','Algorithm','trust-region-reflective','Display','off');
    x = lsqnonlin(F,x0,lb,ub,options);
          
    v = velocity(x,E0,v0,Kf,rho0,rhof);
    r = resistivity(x,Cf(i));

    X = [X;x];

    V_forward = [V_forward;v];

    R_forward = [R_forward;r];

end

function F = JointInvresion(x,Vp_wet_meas,Vs_wet_meas,R_meas,E0,v0,Kf,rho0,rhof,Cf) % crack-pore model

% Velocity
% --------
K0 = E0/(3 * (1 - 2 * v0));
G0 = E0/(2 * (1 + v0));
h = 16/9 * (1 - v0^2)/(1 - v0/2);

cd = x(1);
ar = x(2);

dc = 9/16 * pi * (1 - 2 * v0)/(1 - v0^2) * K0/Kf * ar;

K_wet = K0/(1 + cd * h/(1 - 2 * v0) * (1 + ((1 - v0/2) * dc/(1+dc) - 1)));
G_wet = G0/(1 + cd * h/(1 + v0) * (1 + 2/5 * ((1 - v0/2) * dc/(1+dc) - 1)));

cp = pi * cd * ar;
rho_wet = rhof * cp + rho0 * (1 - cp);

Vp_wet = sqrt((K_wet + 4/3 * G_wet)/rho_wet);
Vs_wet = sqrt(G_wet/rho_wet);
% ---------

% Resistivity
% ---------
p = pi^2/4 * cd;
p(p>1) = 1;
p(p<1/3) = 1/3;
f = 1 - 4 * (sqrt(1./p - 3/4) - 1/2).^3 + 3 * (sqrt(1./p - 3/4) - 1/2).^4;

C = 1/4 .* f .* cp .* Cf;
R = 1./C;

% ----------
dVp = 0.25;
dVs = 0.25;
dR = 0.5;

F = [(Vp_wet_meas   - Vp_wet)/dVp;
     (Vs_wet_meas   - Vs_wet)/dVs;
     (log10(R_meas) - log10(R))/dR];

end

function V = velocity(x,E0,v0,Kf,rho0,rhof)

K0 = E0/(3 * (1 - 2 * v0));
G0 = E0/(2 * (1 + v0));
h = 16/9 * (1 - v0^2)/(1 - v0/2);

cp = pi * x(1) * x(2);

dc = 9/16 * pi * (1 - 2 * v0)/(1 - v0^2) * K0/Kf * ar;

K = K0/(1 + x(1) * h/(1 - 2 * v0) * (1 + ((1 - v0/2) * dc/(1+dc) - 1)));
G = G0/(1 + x(1) * h/(1 + v0) * (1 + 2/5 * ((1 - v0/2) * dc/(1+dc) - 1)));

rho = rhof * cp + rho0 * (1 - cp);

V(1) = sqrt((K + 4/3 * G)/rho);
V(2) = sqrt(G/rho);
end

function R = resistivity(x,Cf)

p = pi^2/4 * x(1);
p(p>1) = 1;
p(p<1/3) = 1/3;
f = 1 - 4 * (sqrt(1./p - 3/4) - 1/2).^3 + 3 * (sqrt(1./p - 3/4) - 1/2).^4;

cp = pi * x(1) * x(2);

C = 1/4 .* f .* cp .* Cf;
R = 1./C;

end
